import React, { useCallback, useState } from "react";

import {
  deleteAnnouncement,
  updateAnnouncementContent,
} from "actions/announcements";
import AnnouncementEditModal from "components/annoucements/AnnouncementEditModal";
import AnnouncementList from "components/annoucements/AnnouncementList";
import ModalConfirm from "components/modals/ModalConfirm";
import { useItemActionConfirm } from "hooks";
import { AnnouncementStore, AuthStore } from "stores";

const AnnoucementsContainer = () => {
  const [itemToEdit, setItemToEdit] = useState(null);

  const [
    itemToDelete,
    setItemToDelete,
    onDeleteConfirm,
    onDeleteCancel,
  ] = useItemActionConfirm(deleteAnnouncement);

  const { isAuthenticated, user } = AuthStore.useState();
  const items = AnnouncementStore.useState((state) =>
    state.itemIds.map((id) => state.items[id])
  );

  const confirmEdit = useCallback(
    async (newContent) => {
      if (itemToEdit && newContent) {
        await updateAnnouncementContent.run({ item: itemToEdit, newContent });
        setItemToEdit(null);
      }
    },
    [itemToEdit, setItemToEdit]
  );

  const cancelEdit = useCallback(() => {
    setItemToEdit(null);
  }, [setItemToEdit]);

  /**
   * Mark down user saw this announcement already.
   * @param {CF2021.Announcement} announcement
   */
  const markSeen = (announcement) => {
    AnnouncementStore.update((state) => {
      state.items[announcement.id].seen = true;
    });
  };

  return (
    <>
      <AnnouncementList
        items={items}
        canRunActions={isAuthenticated && user.role === "chairman"}
        onDelete={setItemToDelete}
        onEdit={setItemToEdit}
        onSeen={markSeen}
      />
      <ModalConfirm
        isOpen={!!itemToDelete}
        onConfirm={onDeleteConfirm}
        onCancel={onDeleteCancel}
        title="Opravdu smazat?"
        yesActionLabel="Smazat"
      >
        Tato akce je nevratná. Opravdu chcete toto oznámení smazat?
      </ModalConfirm>
      {itemToEdit && (
        <AnnouncementEditModal
          isOpen={true}
          announcement={itemToEdit}
          onConfirm={confirmEdit}
          onCancel={cancelEdit}
        />
      )}
    </>
  );
};

export default AnnoucementsContainer;
